/*

Copyright 2001 Ionific Ltd.

This file is part of PlusV Reference Software.

PlusV Reference Software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

PlusV Reference Software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PlusV Reference Software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/**
 * @file        lamepv.c
 * @guid        51dc5d1ceeea48c394b980c1471afb9c
 * @copyright   2001 Ionific Ltd.
 * @author      Jussi Lahdenniemi, jussi.lahdenniemi@ionific.com
 * @project     plusv/encoder
 * @description Wrapper for Lame, for encoding MP3+V bitstreams
 * @note        This app is a quick hack, just to get some MP3+V bitstreams
 *              encoded. Hopefully, MP3 encoder developers will soon
 *              incorporate +V technology inside the encoders (hint, hint)...
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "interleave.h"

#define VERSION "1.0"
#define PCMINTEMP "plusv.temp.in.wav"
#define PCMTEMP "plusv.temp.wav"
#define MP3TEMP "plusv.temp.mp3"
//#define PLUSVTEMP "plusv.temp.pv"

/* Module-local variables */

char *mp3File = "plusv.temp.wav.mp3";
char *outFile = NULL;
char *pvFile = "plusv.temp.pv";

/**
 * @function    main
 * @guid        69e378301caa4c878260142acfad3100
 * @description The main function.
 * @param       argc {int, in} Number of arguments.
 * @param       argv[] {char*, in} Command line arguments.
 * @return      {int} Status
 * @value       Zero if success.
 * @value       Nonzero if failure.
 */
int main( int argc, char* argv[] )
{
    if(chkArgs(argc, argv))
    {
		return 1;
	}
    
	dointerleave();
	
    return 0;
}

int chkArgs(int count, char* args[])
{
	int i; /* counter variable for the loop that checks the arguments */
	/* starts at 1, the first argument */
	for(i = 1; i < count; i++)
	{
		if(args[i][0] == '-') /* if the argument is a parameter, which is it? */
		{
			/* plusV file name */
			if(strcmp(args[i], "-inpv") == 0)
			{
				pvFile = args[++i];
			}
			else
			/* input file name */
			if(strcmp(args[i], "-inmp3") == 0)
			{
				mp3File = args[++i];
				/*printf("-if %s\n", inFile);*/
			}
			else
			/* output file name */
			if(strcmp(args[i], "-of") == 0)
			{
				outFile = args[++i];
				/*printf("-of %s\n", outFile);*/
			}
		}
		else
		{
			printf("-inpv <plusV file>\n");
			printf("-inmp3 <mp3 file>\n");
			printf("-of <output file>\n");
			return 1;
		}
	}
	
	if(count <= 1)
	{
		printf("Interleaves a plusV stream with an mp3 stream.\n");
		printf("Usage: intmp3pv <args>\n");
		printf("  -inpv <plusV file>\n");
		printf("  -inmp3 <mp3 file>\n");
		printf("  -of <output file> (required)\n");
		return 1;
	}
	else
	{
		return 0;
	}
}

/**
 * @function    dointerleave
 * @guid        a6d890b283d64def97db6fa1feb2b332
 * @description Perform interleaving of MP3 and +V streams.
 * @return      {int} Success indicator
 * @value       zero for success
 * @value       nonzero for error
 */
int dointerleave()
{
    FILE* pvin = fopen( pvFile, "rb" );
    FILE* mp3in = fopen( mp3File, "rb" );
    FILE* out = fopen( outFile, "wb" );
    int r = 0;

    if( pvin == NULL || mp3in == NULL )
    {
        fprintf( stderr, "Error opening temporary files!\n" );
        r = 1;
    }
    if( out == NULL )
    {
        fprintf( stderr, "Error opening the output file!\n" );
        r = 1;
    }

    if( r == 0 )
    {
        /* interleave defined in interleave.c */

        interleave( mp3in, pvin, out );
    }

    if( pvin != NULL ) fclose( pvin );
    if( mp3in != NULL ) fclose( mp3in );
    if( out != NULL ) fclose( out );

    return r;
}
