/*

Copyright 1995-2001 Henrik Herranen.

This file is part of PlusV Reference Software.

PlusV Reference Software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

PlusV Reference Software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PlusV Reference Software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <stdio.h>
#include <stdlib.h>
#include "leo_types.h"
#include "leo_endian.h"


lbool BigEndian(void) {
    static lbool virgin = TRUE;
    static lbool result;
    if (virgin) {
	u_int16 test = 0x1371;
	virgin = FALSE;
	if (((char *)&test)[0] == 0x13) {
	    result = TRUE;
	} else if (((char *)&test)[0] == 0x71) {
	    result = FALSE;
	} else {
	    fprintf(stderr, "Internal error: Can't figure out endianness.\n");
	    exit(EXIT_FAILURE);
	}
    }
    return result;
}

u_int32 U32toBE(u_int32 n) {
    if (!BigEndian())
	return (u_int32) U16toBE((u_int16) (n >> 16)) |
	    ((u_int32) U16toBE((u_int16)(n & 65535UL)) << 16);
    else
	return n;
}

s_int32 S32toBE(s_int32 n) {
    return (s_int32) U32toBE((u_int32) n);
}

u_int16 U16toBE(u_int16 n) {
    if (!BigEndian())
	return (u_int16) (((n&0xFF)<<8) | (n>>8));
    else
	return n;
}

s_int16 S16toBE(s_int16 n) {
    return (s_int16) U16toBE((u_int16) n);
}




u_int32 U32toLE(u_int32 n) {
    if (BigEndian())
	return (u_int32) U16toLE((u_int16) (n >> 16)) |
	    ((u_int32) U16toLE((u_int16)(n & 65535UL)) << 16);
    else
	return n;
}

s_int32 L32toLE(s_int32 n) {
    return (s_int32) U32toLE((u_int32) n);
}

u_int16 U16toLE(u_int16 n) {
    if (BigEndian())
	return (u_int16) (((n&0xFF)<<8) | (n>>8));
    else
	return n;
}

s_int16 S16toLE(s_int16 n) {
    return (s_int16) U16toLE((u_int16) n);
}




static void SwitchFLT64(flt64 *x) {
    unsigned char *p = (unsigned char *)x;
    *(p+0) ^= *(p+7);
    *(p+1) ^= *(p+6);
    *(p+2) ^= *(p+5);
    *(p+3) ^= *(p+4);

    *(p+7) ^= *(p+0);
    *(p+6) ^= *(p+1);
    *(p+5) ^= *(p+2);
    *(p+4) ^= *(p+3);

    *(p+0) ^= *(p+7);
    *(p+1) ^= *(p+6);
    *(p+2) ^= *(p+5);
    *(p+3) ^= *(p+4);
}

static void SwitchFLT32(flt32 *x) {
    unsigned char *p = (unsigned char *)x;
    *(p+0) ^= *(p+3);
    *(p+1) ^= *(p+2);

    *(p+3) ^= *(p+0);
    *(p+2) ^= *(p+1);

    *(p+0) ^= *(p+3);
    *(p+1) ^= *(p+2);
}




void FLT64toBE(flt64 *x) {
    if (!BigEndian())
	SwitchFLT64(x);
}

void FLT32toBE(flt32 *x) {
    if (!BigEndian())
	SwitchFLT32(x);
}

void FLT64toLE(flt64 *x) {
    if (BigEndian())
	SwitchFLT64(x);
}

void FLT32toLE(flt32 *x) {
    if (BigEndian())
	SwitchFLT32(x);
}
