/*

Copyright 1995-2001 Henrik Herranen.

This file is part of PlusV Reference Software.

PlusV Reference Software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

PlusV Reference Software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PlusV Reference Software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef LEO_SOUND_H
#define LEO_SOUND_H

#define ALIGN 8
#define LEOF 0x7FFFFFFF

#include "leo_types.h"

/*

   Global structure to be used by all software

 */
struct SAMPLE_HEADER {
    s_int32 channels;
    s_int32 sampleRate;
    s_int32 samples;
    s_int32 bitsPerSample;	/* if negative, then non-linear */
    s_int32 fileType;		/* for those who want to know */
};





/* IFF 8SVX DEFINITIONS */
#define MakeID(a,b,c,d) ((a)<<24 | (b)<<16 | (c)<<8 | (d))

#define ID_FORM MakeID('F','O','R','M')
#define ID_8SVX MakeID('8','S','V','X')
#define ID_VHDR MakeID('V','H','D','R')
#define ID_BODY MakeID('B','O','D','Y')
#define ID_CHAN MakeID('f','m','t',' ')
#define ID_RIFF MakeID('R','I','F','F')
#define ID_WAVE MakeID('W','A','V','E')
#define ID_DATA MakeID('d','a','t','a')
#define ID__FMT MakeID('f','m','t',' ')

struct IFF_8SVX_HEADER {
    u_int32 oneShotHiSamples;
    u_int32 repeatHiSamples;
    u_int32 samplesPerHiCycle;
    u_int16 samplesPerSec;
    u_int8  ctOctave, ctCompression;
    s_int32 volume;
};

#define CHAN_MONO_LEFT 2
#define CHAN_MONO_RIGHT 4
#define CHAN_STEREO 6

struct IFF_8SVX_HEADER_VOICE8CHAN {
    s_int32 chanCode;
};

/* AU DEFINITIONS */
#define ID_SND MakeID('.','s','n','d')


#define FLAG_OUTPUT_ULAW  (1<<0)
#define FLAG_OUTPUT_AUDIO (1<<1)
#define FLAG_LEFT_ONLY    (1<<2)
#define FLAG_RIGHT_ONLY   (1<<3)

struct SUN_AU_HEADER {
    char magic[4];
    u_int32 headerSize;
    u_int32 dataSize;
    u_int32 encoding;
    u_int32 sampleRate;
    u_int32 channels;
};

#define ENC_ULAW  1
#define ENC_S16   3


/* WAV DEFINITIONS */

/* purloined from public Microsoft RIFF docs */

#define	WAVE_FORMAT_UNKNOWN		(0x0000)
#define	WAVE_FORMAT_PCM			(0x0001) 
#define	WAVE_FORMAT_ADPCM		(0x0002)
#define	WAVE_FORMAT_ALAW		(0x0006)
#define	WAVE_FORMAT_MULAW		(0x0007)
#define	WAVE_FORMAT_OKI_ADPCM		(0x0010)
#define	WAVE_FORMAT_DIGISTD		(0x0015)
#define	WAVE_FORMAT_DIGIFIX		(0x0016)
#define	IBM_FORMAT_MULAW         	(0x0101)
#define	IBM_FORMAT_ALAW			(0x0102)
#define	IBM_FORMAT_ADPCM         	(0x0103)

struct RIFF_WAV_HEADER {
    u_int16 formatTag;
    u_int16 channels;
    u_int32 samplesPerSecond;
    u_int32 avgBytesPerSec;
    u_int16 blockAlign;
    u_int16 bitsPerSample;
};


/* LB16 definitions */

#define ID_LB16 MakeID('L','B','1','6')

struct LB16_HEADER {
    u_int32 magic;	/* NOTE! This is dyte order dependent as well as
			   all other values! */
    u_int32 samples;	/* 0xFFFFFFFFU for unknown */
    u_int32 sampleRate;
    u_int8 channels;
    u_int8 flags;
    u_int16 futExp;	/* Don't use! */
};





enum SND_FILETYPE {
	F_URAWB,
	F_SRAWB,
	F_URAWW,
	F_SRAWW,
	F_8SVX,
	F_AUU,
	F_AUW,
	F_ULAW,
	F_WAVU,
	F_WAVB,
	F_WAVW,
	F_WAVL,
	F_LB16,
	F_SRAWLE,
	F_UNKNOWN
};



struct SAMPLE_HEADER *ReadSampleHeaderMagic(FILE *fp,
						enum SND_FILETYPE fileType,
						u_int32 magic);
struct SAMPLE_HEADER *ReadSampleHeader(FILE *fp);
u_int32 ReadSamples(s_int16 *pos, u_int32 n);   /* Range -0x8000..0x7FFF */
u_int32 ReadLSamples(s_int32 *pos, u_int32 n);  /* R -0x80000000..0x7FFFFFFF */
struct SAMPLE_HEADER *WriteSampleHeader(FILE *fp, enum SND_FILETYPE fileType,
				s_int32 channels, s_int32 sampleRate,
				s_int32 samples);
lbool WriteSamples(s_int16 *pos, u_int32 n);  /* Range -0x8000..0x7FFF */
lbool WriteLSamples(s_int32 *pos, u_int32 n); /* R -0x80000000..0x7FFFFFFF */
lbool WriteSampleFooter(void);

#endif
