/*

Copyright 1995-2001 Henrik Herranen

This file is part of PlusV Reference Software.

PlusV Reference Software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

PlusV Reference Software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PlusV Reference Software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef LEO_TYPES_H
#define LEO_TYPES_H

/*

FILE: leo_types.h - defines some types used
DESCRIPTION: g_types.h defines multiple types to ease programming.
  Two special types, LL and ULL are defined to allow easy extension
  to 64-bit instructions and data later. Also system independent
  fixed-size types are included for applications where it is important
  to know exactly the size of a constant.

*/


#define LL32		/* define LL32 if you don't have "long long"'s */



#include <limits.h>


/*

  The following definitions are OBSOLETE. Don't use them in any of your
  future projects!

*/
#if defined _AMIGA || AMIGA
	#include <exec/types.h>
#else
	typedef void		VOID;
	typedef unsigned char	UBYTE;
	typedef signed char	BYTE;
	typedef unsigned short	UWORD;
	typedef short		WORD;
	typedef unsigned long	ULONG;
	typedef long		LONG;
	typedef double		DOUBLE;
	typedef float		FLOAT;
	typedef short		BOOL;
#endif
/*

   End of OBSOLETE definitions.

*/




/*
   For machines with a 64-bit "long long" type.
 */
#ifndef LL32
	typedef long double		LONGDBL;
	typedef unsigned long long      ULL;
	typedef long long               LL;

	#define FMT_LLX			"llx"
	#define FMT_LLD			"lld"
#else
/*
   For machines without a 64-bit "long long" type.
 */
	typedef double			LONGDBL;
	typedef unsigned long		ULL;
	typedef long			LL;

	#define FMT_LLX			"lx"
	#define FMT_LLD			"ld"

	#ifdef LLONG_MIN
		#undef LLONG_MIN
	#endif
	#ifdef LLONG_MAX
		#undef LLONG_MAX
	#endif
	#ifdef ULLONG_MAX
		#undef ULLONG_MAX
	#endif
	#define LLONG_MIN		LONG_MIN
	#define LLONG_MAX		LONG_MAX
	#define ULLONG_MAX		ULONG_MAX

#endif

ULL StrToULL(const char *str, char **ptr, int base);
LL  StrToLL(const char *str, char **ptr, int base);

#define WORDMASK(b)		(((ULL)-1)>>(sizeof(ULL)*8-(b)))

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif



/*

   System-independent fixed-sized integers and floating point types.

 */
#if CHAR_MAX == 127
	typedef char		s_int8;
#else
	typedef signed char	s_int8;
#endif
#define FMT_S8 "%d"
#if CHAR_MAX == 255
	typedef char		u_int8;
#else
	typedef unsigned char	u_int8;
#endif
#define FMT_U8 "%d"
typedef short			s_int16;
#define FMT_S16 "%d"
typedef unsigned short		u_int16;
#define FMT_U16 "%d"
typedef int			s_int32;
#define FMT_S32 "%d"
typedef unsigned int		u_int32;
#define FMT_U32 "%u"
typedef float			flt32;
#define FMT_F32 "%f"
typedef double			flt64;
#define FMT_F64 "%lf"
/*
   Some more types that may come handy, but without a fixed size.
 */
typedef int			lbool;


/*

   Fix for broken <stdlib.h>

 */
#ifndef EXIT_SUCCESS
	#define EXIT_SUCCESS		0
#endif
#ifndef EXIT_FAILURE
	#define EXIT_FAILURE		1
#endif

#endif
