/*

Copyright 2001 VLSI Solution Oy.

This file is part of PlusV Reference Software.

PlusV Reference Software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

PlusV Reference Software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PlusV Reference Software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*

  plusmain.c

  This is the main program for the PlusV encoder / decoder. It requires that
  MP3 encoding and decoding has been done on the outside, and it also requires
  a separate program for interleaving PlusV data to the MP3 stream. Thus, this
  program doesn't contain an MP3 encoder / decoder, and it may also be used in
  conjunction with other compression formats.

  (C) 2001 VLSI Solution OY, http://www.vlsi.fi/
  Author: Henrik Herranen

  This reference source code may be used freely as long as the PlusV format
  is not intentionally broken!

  */


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "leo_types.h"
#include "leo_sound.h"
#include "leo_endian.h"
#include "bitstream.h"
#include "filter.h"
#include "plusv.h"

int pvbitspersec;

int main(int argc, char **argv) {
  u_int16 i;
  s_int16 emptyParamNo = 0;
  const char *inName = NULL, *outName = NULL, *plusvName = NULL;
  enum SND_FILETYPE outFileType = F_WAVW;
  s_int16 decode = 0, resample32 = 0;
  int lowBand = -1;
  int highBand = 7;
  int delay = 0;
  int quant = 0;


  /*

      Handle arguments

  */

  for (i=1; i<argc; i++) {
    if (!strcmp(argv[i], "-l")) {
      outFileType = F_WAVL;
    } else if (!strcmp(argv[i], "-w")) {
      outFileType = F_WAVW;
    } else if (!strcmp(argv[i], "-u")) {
      outFileType = F_WAVU;
    } else if (!strcmp(argv[i], "-d")) {
      decode = 1;
    } else if (!strcmp(argv[i], "-e")) {
      decode = 0;
    } else if (!strcmp(argv[i], "-b")) {
      if (++i >= argc) {
	fprintf(stderr, "A value must follow option \"%s\"\n", argv[i-1]);
	exit(EXIT_FAILURE);
      }
      lowBand = atoi(argv[i]);
    } else if (!strcmp(argv[i], "-B")) {
      if (++i >= argc) {
	fprintf(stderr, "A value must follow option \"%s\"\n", argv[i-1]);
	exit(EXIT_FAILURE);
      }
      highBand = atoi(argv[i]);
    } else if (!strcmp(argv[i], "-D")) {
      if (++i >= argc) {
	fprintf(stderr, "A value must follow option \"%s\"\n", argv[i-1]);
	exit(EXIT_FAILURE);
      }
      delay = atoi(argv[i]);
    } else if (!strcmp(argv[i], "-q")) {
      if (++i >= argc) {
	fprintf(stderr, "A value must follow option \"%s\"\n", argv[i-1]);
	exit(EXIT_FAILURE);
      }
      quant = atoi(argv[i]);
    } else if (!strcmp(argv[i], "-dd")) {
      decode = 0;
    } else if (!strcmp(argv[i], "-r")) {
      resample32 = 1;
    } else if (!strcmp(argv[i], "-R")) {
      resample32 = 2;
    } else if (!strcmp(argv[i], "-h")) {
      printf("The Mp3+V audio extender v1.01, "
	     "2000-2001 VLSI Solution OY\n"
	     "Author: Henrik 'Leopold' Herranen\n"
	     "  http://www.plusv.org/\n"
		 "Fixed/Modified by: gameplaya15143, " __DATE__ "\n\n"
	     "Usage:\n%s [options] iName oName [pvName]\n\n"
	     "-e\tEncode a PCM to PlusV+PCM files (default=true)\n"
	     "-d\tDecode PlusV+PCM files to PCM\n"
	     "-r\tResample source data to 32 kHz\n"
	     "-R\tDon't resample source data to 32 kHz (default)\n"
	     "-u\tWrite 8-bit ULAW AU format (NOT recommended)\n"
	     "-w\tWrite 16-bit WAV format (default)\n"
	     "-l\tWrite 32-bit WAV format\n"
	     "-b band\tLow band for full-rate encoding (-1=half, 0..7=full)\n"
	     "-B band\tHigh band (1..7, default=7)\n"	// [gameplaya15143] added this option
	     "-D int\tDelay for interleaving with an mp3 (default=0)\n"	// [gameplaya15143] added this. woo hoo, it works!
	     "-q int\tEncode quant (0 = smallest, 2 = best, default=0)\n"	// [gameplaya15143] added this option
	     "-h\tThis help page\n"
	     "iName\tInput file name\n"
	     "oName\tOutput file name\n"
	     "pvName\tPlusV file name. If omitted, standard output/input is\n"
	     "\tused for encoding/decoding respectively.\n"
	     "\n", argv[0]);
      return EXIT_SUCCESS;
    } else {
      switch(emptyParamNo++) {
      case 0:
	inName = argv[i];
	break;
      case 1:
	outName = argv[i];
	break;
      case 2:
	plusvName = argv[i];
	break;
      default:
	fprintf(stderr,
		"%s: File name \"%s\" extraneous\n",
		argv[0], argv[i]);
	exit(EXIT_FAILURE);
      }
    }
  }

  if (!outName) {
    fprintf(stderr, "%s: You have to define both an input and output name.\n"
	    "\tTry option \"-h\" for details.\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  return decode ?
    DecodeFile(inName, outName, plusvName, outFileType) :
    EncodeFile(inName, outName, plusvName, outFileType, resample32, lowBand, highBand, quant, delay);
}
