/*

Copyright 2001 VLSI Solution Oy.

This file is part of PlusV Reference Software.

PlusV Reference Software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

PlusV Reference Software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PlusV Reference Software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "pvrandom.h"

unsigned long random_state = 0x16a5bce3;

/*
  pvrandom() returns a 31-bit random number. The random generator seed value
  goes around every 2^31-2 calls. Note, than the modulo operator usually found
  in random number algorithms is replaced with some multiplications to make
  the code more efficient to implement in a DSP processor.
 */

long pvrandom(void) {
  unsigned short hi;
  long lo;
  /* 32 bits = (32 bits * 32 bits) >> 47 bits. */
#ifdef _MSC_VER
  /* MSVC does not support the ULL suffix */
  hi = (unsigned short)(((unsigned __int64)random_state*(unsigned __int64)0x41A705B0) >> 47 );
#else
  hi = (random_state*0x41A705B0ULL) >> 47;
#endif
  lo = random_state - hi * 127773L;
  lo = 16807 * lo - 2836L * hi;
  if (lo <= 0)
    lo += 0x7fffffff;
  return random_state = (lo & 0x7fffffff);
}

void pvsrandom(unsigned long x) {
  random_state = x;
}

